package com.sun.security.ntlm;

import java.util.Locale;
import java.io.UnsupportedEncodingException;
import javax.crypto.spec.SecretKeySpec;
import java.util.Arrays;
import javax.crypto.SecretKey;
import javax.crypto.BadPaddingException;
import javax.crypto.IllegalBlockSizeException;
import java.security.spec.InvalidKeySpecException;
import java.security.InvalidKeyException;
import javax.crypto.spec.DESKeySpec;
import java.io.IOException;
import sun.misc.HexDumpEncoder;
import java.security.NoSuchAlgorithmException;
import javax.crypto.NoSuchPaddingException;
import sun.security.provider.MD4;
import javax.crypto.Mac;
import java.security.MessageDigest;
import javax.crypto.Cipher;
import javax.crypto.SecretKeyFactory;

class NTLM
{
    private final SecretKeyFactory fac;
    private final Cipher cipher;
    private final MessageDigest md4;
    private final Mac hmac;
    private final MessageDigest md5;
    private static final boolean DEBUG;
    final Version v;
    final boolean writeLM;
    final boolean writeNTLM;
    
    protected NTLM(String str) throws NTLMException {
        if (str == null) {
            str = "LMv2/NTLMv2";
        }
        int n = -1;
        switch (str.hashCode()) {
            case 2433: {
                if (str.equals("LM")) {
                    n = 0;
                    break;
                }
                break;
            }
            case 2406855: {
                if (str.equals("NTLM")) {
                    n = 1;
                    break;
                }
                break;
            }
            case 981059989: {
                if (str.equals("LM/NTLM")) {
                    n = 2;
                    break;
                }
                break;
            }
            case 74612555: {
                if (str.equals("NTLM2")) {
                    n = 3;
                    break;
                }
                break;
            }
            case 2341821: {
                if (str.equals("LMv2")) {
                    n = 4;
                    break;
                }
                break;
            }
            case -1981975933: {
                if (str.equals("NTLMv2")) {
                    n = 5;
                    break;
                }
                break;
            }
            case -1885054699: {
                if (str.equals("LMv2/NTLMv2")) {
                    n = 6;
                    break;
                }
                break;
            }
        }
        switch (n) {
            case 0: {
                this.v = Version.NTLM;
                this.writeLM = true;
                this.writeNTLM = false;
                break;
            }
            case 1: {
                this.v = Version.NTLM;
                this.writeLM = false;
                this.writeNTLM = true;
                break;
            }
            case 2: {
                this.v = Version.NTLM;
                boolean b = true;
                this.writeNTLM = b;
                this.writeLM = b;
                break;
            }
            case 3: {
                this.v = Version.NTLM2;
                boolean b2 = true;
                this.writeNTLM = b2;
                this.writeLM = b2;
                break;
            }
            case 4: {
                this.v = Version.NTLMv2;
                this.writeLM = true;
                this.writeNTLM = false;
                break;
            }
            case 5: {
                this.v = Version.NTLMv2;
                this.writeLM = false;
                this.writeNTLM = true;
                break;
            }
            case 6: {
                this.v = Version.NTLMv2;
                boolean b3 = true;
                this.writeNTLM = b3;
                this.writeLM = b3;
                break;
            }
            default: {
                throw new NTLMException(5, "Unknown version " + str);
            }
        }
        try {
            this.fac = SecretKeyFactory.getInstance("DES");
            this.cipher = Cipher.getInstance("DES/ECB/NoPadding");
            this.md4 = MD4.getInstance();
            this.hmac = Mac.getInstance("HmacMD5");
            this.md5 = MessageDigest.getInstance("MD5");
        }
        catch (NoSuchPaddingException ex) {
            throw new AssertionError();
        }
        catch (NoSuchAlgorithmException ex2) {
            throw new AssertionError();
        }
    }
    
    public void debug(String format, Object... args) {
        if (NTLM.DEBUG) {
            System.out.printf(format, args);
        }
    }
    
    public void debug(byte[] array) {
        if (NTLM.DEBUG) {
            try {
                new HexDumpEncoder().encodeBuffer(array, System.out);
            }
            catch (IOException ex) {}
        }
    }
    
    byte[] makeDesKey(byte[] array, int n) {
        int[] array2 = new int[array.length];
        for (int i = 0; i < array2.length; ++i) {
            array2[i] = ((array[i] < 0) ? (array[i] + 256) : array[i]);
        }
        return new byte[] { (byte)array2[n + 0], (byte)((array2[n + 0] << 7 & 0xFF) | array2[n + 1] >> 1), (byte)((array2[n + 1] << 6 & 0xFF) | array2[n + 2] >> 2), (byte)((array2[n + 2] << 5 & 0xFF) | array2[n + 3] >> 3), (byte)((array2[n + 3] << 4 & 0xFF) | array2[n + 4] >> 4), (byte)((array2[n + 4] << 3 & 0xFF) | array2[n + 5] >> 5), (byte)((array2[n + 5] << 2 & 0xFF) | array2[n + 6] >> 6), (byte)(array2[n + 6] << 1 & 0xFF) };
    }
    
    byte[] calcLMHash(byte[] array) {
        byte[] array2 = { 75, 71, 83, 33, 64, 35, 36, 37 };
        byte[] array3 = new byte[14];
        int length = array.length;
        if (length > 14) {
            length = 14;
        }
        System.arraycopy(array, 0, array3, 0, length);
        try {
            DESKeySpec keySpec = new DESKeySpec(this.makeDesKey(array3, 0));
            DESKeySpec keySpec2 = new DESKeySpec(this.makeDesKey(array3, 7));
            SecretKey generateSecret = this.fac.generateSecret(keySpec);
            SecretKey generateSecret2 = this.fac.generateSecret(keySpec2);
            this.cipher.init(1, generateSecret);
            byte[] doFinal = this.cipher.doFinal(array2, 0, 8);
            this.cipher.init(1, generateSecret2);
            byte[] doFinal2 = this.cipher.doFinal(array2, 0, 8);
            byte[] array4 = new byte[21];
            System.arraycopy(doFinal, 0, array4, 0, 8);
            System.arraycopy(doFinal2, 0, array4, 8, 8);
            return array4;
        }
        catch (InvalidKeyException ex) {
            assert false;
        }
        catch (InvalidKeySpecException ex2) {
            assert false;
        }
        catch (IllegalBlockSizeException ex3) {
            assert false;
        }
        catch (BadPaddingException ex4) {
            assert false;
        }
        return null;
    }
    
    byte[] calcNTHash(byte[] input) {
        byte[] digest = this.md4.digest(input);
        byte[] array = new byte[21];
        System.arraycopy(digest, 0, array, 0, 16);
        return array;
    }
    
    byte[] calcResponse(byte[] array, byte[] input) {
        try {
            assert array.length == 21;
            DESKeySpec keySpec = new DESKeySpec(this.makeDesKey(array, 0));
            DESKeySpec keySpec2 = new DESKeySpec(this.makeDesKey(array, 7));
            DESKeySpec keySpec3 = new DESKeySpec(this.makeDesKey(array, 14));
            SecretKey generateSecret = this.fac.generateSecret(keySpec);
            SecretKey generateSecret2 = this.fac.generateSecret(keySpec2);
            SecretKey generateSecret3 = this.fac.generateSecret(keySpec3);
            this.cipher.init(1, generateSecret);
            byte[] doFinal = this.cipher.doFinal(input, 0, 8);
            this.cipher.init(1, generateSecret2);
            byte[] doFinal2 = this.cipher.doFinal(input, 0, 8);
            this.cipher.init(1, generateSecret3);
            byte[] doFinal3 = this.cipher.doFinal(input, 0, 8);
            byte[] array2 = new byte[24];
            System.arraycopy(doFinal, 0, array2, 0, 8);
            System.arraycopy(doFinal2, 0, array2, 8, 8);
            System.arraycopy(doFinal3, 0, array2, 16, 8);
            return array2;
        }
        catch (IllegalBlockSizeException ex) {
            assert false;
        }
        catch (BadPaddingException ex2) {
            assert false;
        }
        catch (InvalidKeySpecException ex3) {
            assert false;
        }
        catch (InvalidKeyException ex4) {
            assert false;
        }
        return null;
    }
    
    byte[] hmacMD5(byte[] original, byte[] input) {
        try {
            this.hmac.init(new SecretKeySpec(Arrays.copyOf(original, 16), "HmacMD5"));
            return this.hmac.doFinal(input);
        }
        catch (InvalidKeyException ex) {
            assert false;
        }
        catch (RuntimeException ex2) {
            assert false;
        }
        return null;
    }
    
    byte[] calcV2(byte[] array, String s, byte[] array2, byte[] array3) {
        try {
            byte[] hmacMD5 = this.hmacMD5(array, s.getBytes("UnicodeLittleUnmarked"));
            byte[] array4 = new byte[array2.length + 8];
            System.arraycopy(array3, 0, array4, 0, 8);
            System.arraycopy(array2, 0, array4, 8, array2.length);
            byte[] array5 = new byte[16 + array2.length];
            System.arraycopy(this.hmacMD5(hmacMD5, array4), 0, array5, 0, 16);
            System.arraycopy(array2, 0, array5, 16, array2.length);
            return array5;
        }
        catch (UnsupportedEncodingException ex) {
            assert false;
            return null;
        }
    }
    
    static byte[] ntlm2LM(byte[] original) {
        return Arrays.copyOf(original, 24);
    }
    
    byte[] ntlm2NTLM(byte[] array, byte[] array2, byte[] original) {
        byte[] copy = Arrays.copyOf(original, 16);
        System.arraycopy(array2, 0, copy, 8, 8);
        return this.calcResponse(array, Arrays.copyOf(this.md5.digest(copy), 8));
    }
    
    static byte[] getP1(char[] value) {
        try {
            return new String(value).toUpperCase(Locale.ENGLISH).getBytes("ISO8859_1");
        }
        catch (UnsupportedEncodingException ex) {
            return null;
        }
    }
    
    static byte[] getP2(char[] value) {
        try {
            return new String(value).getBytes("UnicodeLittleUnmarked");
        }
        catch (UnsupportedEncodingException ex) {
            return null;
        }
    }
    
    static {
        DEBUG = (System.getProperty("ntlm.debug") != null);
    }
    
    static class Reader
    {
        private final byte[] internal;
        
        Reader(byte[] internal) {
            this.internal = internal;
        }
        
        int readInt(int n) throws NTLMException {
            try {
                return (this.internal[n] & 0xFF) + ((this.internal[n + 1] & 0xFF) << 8) + ((this.internal[n + 2] & 0xFF) << 16) + ((this.internal[n + 3] & 0xFF) << 24);
            }
            catch (ArrayIndexOutOfBoundsException ex) {
                throw new NTLMException(1, "Input message incorrect size");
            }
        }
        
        int readShort(int n) throws NTLMException {
            try {
                return (this.internal[n] & 0xFF) + (this.internal[n + 1] & 0xFF00);
            }
            catch (ArrayIndexOutOfBoundsException ex) {
                throw new NTLMException(1, "Input message incorrect size");
            }
        }
        
        byte[] readBytes(int from, int n) throws NTLMException {
            try {
                return Arrays.copyOfRange(this.internal, from, from + n);
            }
            catch (ArrayIndexOutOfBoundsException ex) {
                throw new NTLMException(1, "Input message incorrect size");
            }
        }
        
        byte[] readSecurityBuffer(int n) throws NTLMException {
            int int1 = this.readInt(n + 4);
            if (int1 == 0) {
                return new byte[0];
            }
            try {
                return Arrays.copyOfRange(this.internal, int1, int1 + this.readShort(n));
            }
            catch (ArrayIndexOutOfBoundsException ex) {
                throw new NTLMException(1, "Input message incorrect size");
            }
        }
        
        String readSecurityBuffer(int n, boolean b) throws NTLMException {
            byte[] securityBuffer = this.readSecurityBuffer(n);
            try {
                if (securityBuffer == null) {
                    return null;
                }
                String s2;
                String s = s2;
                s2 = new String(securityBuffer, b ? "UnicodeLittleUnmarked" : "ISO8859_1");
                return s;
            }
            catch (UnsupportedEncodingException ex) {
                throw new NTLMException(1, "Invalid input encoding");
            }
        }
    }
    
    static class Writer
    {
        private byte[] internal;
        private int current;
        
        Writer(int n, int current) {
            assert current < 256;
            this.internal = new byte[256];
            this.current = current;
            System.arraycopy(new byte[] { 78, 84, 76, 77, 83, 83, 80, 0, (byte)n }, 0, this.internal, 0, 9);
            return;
        }
        
        void writeShort(int n, int n2) {
            this.internal[n] = (byte)n2;
            this.internal[n + 1] = (byte)(n2 >> 8);
        }
        
        void writeInt(int n, int n2) {
            this.internal[n] = (byte)n2;
            this.internal[n + 1] = (byte)(n2 >> 8);
            this.internal[n + 2] = (byte)(n2 >> 16);
            this.internal[n + 3] = (byte)(n2 >> 24);
        }
        
        void writeBytes(int n, byte[] array) {
            System.arraycopy(array, 0, this.internal, n, array.length);
        }
        
        void writeSecurityBuffer(int n, byte[] array) {
            if (array == null) {
                this.writeShort(n + 4, this.current);
            }
            else {
                int length = array.length;
                if (this.current + length > this.internal.length) {
                    this.internal = Arrays.copyOf(this.internal, this.current + length + 256);
                }
                this.writeShort(n, length);
                this.writeShort(n + 2, length);
                this.writeShort(n + 4, this.current);
                System.arraycopy(array, 0, this.internal, this.current, length);
                this.current += length;
            }
        }
        
        void writeSecurityBuffer(int n, String s, boolean b) {
            try {
                this.writeSecurityBuffer(n, (s == null) ? null : s.getBytes(b ? "UnicodeLittleUnmarked" : "ISO8859_1"));
            }
            catch (UnsupportedEncodingException ex) {
                assert false;
            }
        }
        
        byte[] getBytes() {
            return Arrays.copyOf(this.internal, this.current);
        }
    }
}
